#!/usr/bin/env python

import sys


current_lsh_clusterid = None
cluster_count = 1
current_minhash = None

for line in sys.stdin:
    # remove leading and trailing whitespace
    line = line.strip()

    (lsh_cluster_id, minhash) = line.split('\t', 1)

    # this IF-switch only works because HADOOP sorts map output
    # by key (here: word) before it is passed to the reducer
    if current_lsh_clusterid == lsh_cluster_id:
        cluster_count += 1
    else:
        if current_lsh_clusterid:
            (lsh, cluster_id) = current_lsh_clusterid.split("$$$$", 1)
            print "%s\t%s\t%s\t%s" % (lsh, cluster_id, cluster_count, current_minhash)
        cluster_count = 1
        current_lsh_clusterid = lsh_cluster_id
        current_minhash = minhash

if current_lsh_clusterid:
    (lsh, cluster_id) = current_lsh_clusterid.split("$$$$", 1)
    print "%s\t%s\t%s\t%s" % (lsh, cluster_id, cluster_count, current_minhash)



#!/usr/bin/env python

from pyspark import SparkContext
import sys



if __name__ == "__main__":
    sc = SparkContext(appName="SequenceToText")
    in_file = sc.sequenceFile( sys.argv[1])
    print "len:", len(sys.argv)
    if len(sys.argv) >= 4 and sys.argv[3] == "--values":
        values = in_file.map(lambda (x, y): y)
    else:
        values = in_file
    values.saveAsTextFile(sys.argv[2])



#!/usr/bin/env python

import sys

current_key = None

# input comes from STDIN
for line in sys.stdin:
    # remove leading and trailing whitespace
    line = line.strip()

    # parse the input we got from mapper.py
    (key, score) = line.split('\t', 1)

    # this IF-switch only works because Hadoop sorts map output
    # by key (here: word) before it is passed to the reducer
    if current_key == key:
        continue
    else:
        (key1, key2) = key.split("$$$$", 1)
        print '%s\t%s\t%s' % (key1, key2, score)
        current_key = key

exit(0)



#!/usr/bin/env python

from pyspark import SparkContext
from optparse import OptionParser
import json
import hashlib
from pyspark import StorageLevel

class Clusterer:
    def __init__(self, **options):
        self.numPartitions = options.get("numPartitions",10)
        self.computeSimilarity = options.get("computeSimilarity",False)
        self.threshold = options.get("threshold",0.0)
        self.computeIdenticalClusters = options.get("computeIdenticalClusters",False)
        self.base = options.get("base","")
        self.topk = options.get("topk","3")
        self.candidates_name = options.get("candidatesName","candidates")
        #print 'in clustering'
        print self.computeSimilarity,self.threshold,self.topk

    def perform(self,rdd):
        print self.computeSimilarity,self.threshold,self.base
        if len(self.base) > 0:
            base = sc.sequenceFile(self.base).mapValues(lambda x: json.loads(x))
            result = clusterer.compute_clusters_with_base(rdd, base)
        else:
            if self.computeIdenticalClusters is True:
                (key_clusterids, result) = clusterer.compute_identical_clusters(rdd)
            else:
                result = self.compute_clusters(rdd)
        result = self.output_json(result, self.topk, self.candidates_name)
        if self.computeIdenticalClusters is True:
            return (result,key_clusterids)
        else:
            return result


    def compute_clusters_with_base(self, data, base):
        lsh_clusters = data.join(base, self.numPartitions)
        clusters_with_dups = lsh_clusters.flatMap(lambda x: self.__output_clusters_with_base(x[0],list(x[1])))
        return clusters_with_dups.reduceByKey(lambda value1, value2: self.__remove_duplicates(value1, value2))

    def compute_clusters(self, data):
        lsh_clusters = data.partitionBy(self.numPartitions).groupByKey(self.numPartitions)
        clusters_with_dups = lsh_clusters.flatMap(lambda x: self.__output_cluster(x[0], list(x[1])))
        return clusters_with_dups.reduceByKey(lambda value1, value2: self.__remove_duplicates(value1, value2),numPartitions=self.numPartitions)

    def compute_identical_clusters(self, data):
        lsh_clusters = data.groupByKey(self.numPartitions)
        lsh_clusters.persist(StorageLevel.MEMORY_AND_DISK)
        key_clusters = lsh_clusters.flatMap(lambda x: self.__output_key_cluster_ids(x[0], list(x[1])))
        clusterid_clusters = lsh_clusters.flatMap(lambda x: self.__output_cluster_ids_minhash(x[0], list(x[1])))
        clusters_with_dups = clusterid_clusters.groupByKey().flatMap(lambda x: self.__output_cluster(x[0], list(x[1])))
        return key_clusters, self.__deduplicate_clusters(clusters_with_dups)

    def output_csv(self, data, top_k, separator):
        sorted_data = data.mapValues(lambda x: self.__sort_by_score(x))
        if self.computeSimilarity is True and top_k != -1:
            top_data = sorted_data.mapValues(lambda x: self.__get_top_k(x, top_k))
        else:
            top_data = sorted_data
        return top_data.flatMap(lambda x: list(self.__generate_csv(x[0], x[1], separator)))

    def output_json(self, data, top_k, candidates_name):
        if self.computeSimilarity is True:
            sorted_data = data.mapValues(lambda x: self.__sort_by_score(x))
        else:
            sorted_data = data
        if self.computeSimilarity is True and top_k != -1:
            top_data = sorted_data.mapValues(lambda x: self.__get_top_k(x, top_k))
        else:
            top_data = sorted_data
        return top_data.map(lambda x: self.__generate_json(x[0], x[1], candidates_name))

    def __sort_by_score(self, matches):
        return sorted(matches, key=lambda x: float(x[1]), reverse=True)

    def __get_top_k(self, sorted_matches, top_k):
        result = []
        current_score = 0
        for match in sorted_matches:
            score = match[1]
            if len(result) < top_k:
                result.append(match)
                current_score = score
            elif score == current_score:
                result.append(match)
            else:
                break
        return result

    def __generate_json(self, key, matches, candidates_name):
        if len(self.base) > 0:
            json_obj = {"uri": str(key), candidates_name:[]}
            for match in matches:
                # print "Match:", type(match), ", ", match
                candidate = {}
                if type(match) is list or type(match) is dict or type(match) is tuple:
                    candidate["uri"] = str(match[0])
                    candidate["score"] = match[1]
                else:
                    candidate["uri"] = str(match)
                json_obj[candidates_name].append(candidate)
        else:
            json_obj = {"member":[]}
            json_obj["member"].append({"uri":key})
            for match in matches:
                # print "Match:", type(match), ", ", match
                candidate = {}
                if type(match) is list or type(match) is dict or type(match) is tuple:
                    candidate["uri"] = str(match[0])
                    if self.computeSimilarity is True:
                        candidate["score"] = match[1]
                else:
                    candidate["uri"] = str(match)
                json_obj["member"].append(candidate)


        return key + "/cluster", json_obj


    def __generate_csv(self, key, matches, separator):
        if self.computeSimilarity:
            for match in matches:
                if type(match) is list or type(match) is dict or type(match) is tuple:
                    yield str(key) + separator + str(match[0]) + separator + str(match[1])
                else:
                    yield str(key) + separator + str(match)
        else:
            line = str(key)
            for match in matches:
                if type(match) is list or type(match) is dict or type(match) is tuple:
                    line = line + separator + str(match[0])
                else:
                    line = line + separator + str(match)
            yield line


    def __output_clusters_with_base(self, lsh_key, cluster):
        #print "Output clusters for key:", lsh_key
        data = cluster[0]
        key = data[0]
        matches = cluster[1:]
        for match in matches:
            match_key = match[0]
            if self.computeSimilarity is True:
                score = self.__compute_list_similarity_score(data[1], match[1])
                if self.threshold <= 0 or score >= self.threshold:
                    yield key, [(match_key, score)]
            else:
                yield key, [(match_key, 0)]

    def __output_cluster(self, lsh_key, cluster):
        if len(cluster) > 0 :
            for data in cluster:
                key1 = data[0]
                for match in cluster:
                    key2 = match[0]
                    if key1 < key2:
                        if self.computeSimilarity is True:
                            score = self.__compute_list_similarity_score(data[1], match[1])
                            if self.threshold <= 0 or score >= self.threshold:
                                yield key1, [(key2, score)]
                        else:
                            # yield key1, [(key2, 0)]
                            key2_set = set()
                            key2_set.add(key2)
                            yield key1, key2_set

    def __output_key_cluster_ids(self, lsh_key, cluster):
        if len(cluster) > 0 :
            for data in cluster:
                key = data[0]
                hashes = data[1]
                cluster_id = hashlib.sha1(",".join(hashes)).hexdigest()
                yield key, cluster_id

    def __output_cluster_ids_minhash(self, lsh_key, cluster):
        if len(cluster) > 0 :
            for data in cluster:
                key = data[0]
                hashes = data[1]
                cluster_id = hashlib.sha1(",".join(hashes)).hexdigest()
                yield lsh_key, (cluster_id, hashes)

    def __compute_list_similarity_score(self, list1, list2):
        similarity = float(len(set(list1) & set(list2)))/float(len(set(list1)))
        return similarity

    def __remove_duplicates(self, value1, value2):
        if value1 is None:
            return value2
        if value2 is None:
            return value1

        if self.computeSimilarity is True:
            seen = list()
            for i in range(0, len(value1)):
                match = value1[i]
                seen.append(str(match[0]))

            for i in range(0, len(value2)):
                match = value2[i]
                key = str(match[0])
                if key in seen:
                    idx = seen.index(key)

                    score = value1[idx][1]
                    this_score = match[1]

                    if this_score > score:
                        value1[idx] = (key, this_score)
                    continue
                #print "Added new:", key, "seen:", seen
                seen.append(key)
                value1.append(match)
        else:
            return value1.union(value2)

        return value1

if __name__ == "__main__":
    """
        Usage: clusterer.py [input1] [input1Prefix] [base] [base2Prefix] [outputFilename]
    """
    sc = SparkContext(appName="LSH-CLUSTERER")

    usage = "usage: %prog [options] input1 input1Prefix <input2> <input2Prefix> output"
    parser = OptionParser()
    parser.add_option("-r", "--separator", dest="separator", type="string",
                      help="field separator", default="\t")

    parser.add_option("-s", "--computeSimilarity", action="store_true",
                      dest="computeSimilarity", default=False, help="compute similarity")
    parser.add_option("-j", "--computeIdenticalClusters", action="store_true",
                      dest="computeIdenticalClusters", default=False, help="compute identical clusters")
    parser.add_option("-t", "--threshold", type="float",
                      dest="threshold", default=0.0, help="similarity threshold")
    parser.add_option("-e", "--base", dest="base", type="string",
                      help="base file", default="")
    parser.add_option("-o", "--outputformat", dest="outputformat", type="string",
                      help="output file format: text/sequence", default="text")
    parser.add_option("-y", "--outputtype", dest="outputtype", type="string",
                      help="output type: csv/json", default="json")
    parser.add_option("-k", "--topk", dest="topk", type="int",
                      help="top n matches", default=3)
    parser.add_option("-x", "--numPartitions", dest="numPartitions", type="int",
                      help="number of partitions", default=10)
    parser.add_option("-z", "--candidatesName", dest="candidates_name", type="string",
                        help="name for json element for matching candidates", default="candidates")
    (c_options, args) = parser.parse_args()
    print "Got options:", c_options

    inputFilename = args[0]
    outputFilename = args[1]
    print "Save to:", outputFilename

    clusterer = Clusterer(c_options.numPartitions,
                          c_options.computeSimilarity, c_options.threshold)
    rdd = sc.sequenceFile(inputFilename).mapValues(lambda x: json.loads(x))
    if len(c_options.base) > 0:
        base = sc.sequenceFile(c_options.base).mapValues(lambda x: json.loads(x))
        result = clusterer.compute_clusters_with_base(rdd, base)
    else:
        if c_options.computeIdenticalClusters is True:
            (key_clusterids, result) = clusterer.compute_identical_clusters(rdd)
        else:
            result = clusterer.compute_clusters(rdd)

    if c_options.outputtype == "json":
        result = clusterer.output_json(result, c_options.topk, c_options.candidates_name)
    else:
        result = clusterer.output_csv(result, c_options.topk, c_options.separator)

    if c_options.outputformat == "text":
        result.saveAsTextFile(outputFilename)
        if c_options.computeIdenticalClusters is True:
            key_clusterids.saveAsTextFile(outputFilename + "-key-clusterids")
    else:
        result.mapValues(lambda x: json.dumps(x)).saveAsSequenceFile(outputFilename)
        if c_options.computeIdenticalClusters is True:
            key_clusterids.mapValues(lambda x, y: json.dumps(x)).saveAsSequenceFile(outputFilename + "-key-clusterids")





__author__ = 'rajagopal067'


import sys
import fileinput
import os

sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from utils import util


#This class has computesimilarity method which computes the similarity between files having 2 different prefixes

class ClusterWithBaseGenerator:
    def __init__(self):
        self.separator = "\t"
        self.scorethreshold = 0.0
        self.outputFilenames = []
        self.numRowsPerFile = 1000000

    def __openFileForWrite(self, filehandle):
        print "Open file: ", filehandle
        sys.stdout.flush()
        return open(filehandle, 'w')

    def __generateNextOutputfile(self, outputFilename):
        file_idx = len(self.outputFilenames)
        out_filename = outputFilename + "." + str(file_idx)
        self.outputFilenames.append(out_filename)
        out = self.__openFileForWrite(out_filename)
        return out

    def run(self, inputFilename, baseFilename, outputFilename, separator, computeSimilarity, scoreThreshold):
        self.separator = separator
        self.scorethreshold = scoreThreshold
        lsh_key = None
        lsh_band = None
        prev_lsh_key = None
        prev_lsh_band = None
        currentClusterInput = []
        currentClusterBase = []

        file_list = [inputFilename, baseFilename]
        tmp_combined_file = inputFilename + "tmp" + ".csv"
        with open(tmp_combined_file, 'w') as file:
           input_lines = fileinput.input(file_list)
           file.writelines(input_lines)

#sort the combined file and then compute similarity between keys in the same band having same lsh keys
        util.sort_csv_file(tmp_combined_file, [0], '\t')
        file = open(tmp_combined_file,'r')

        #file = open(inputFilename, 'r')

        del self.outputFilenames[:]

        out = self.__openFileForWrite(outputFilename)

        for line in file:
            line = line.strip()
            if len(line) > 0:
                lineTokens = line.split(separator)
                lsh_key = lineTokens[0]
                lsh_band = lsh_key[0:3]

                itemId = lineTokens[1]

                if prev_lsh_key is None:
                    prev_lsh_key = lsh_key
                    prev_lsh_band = lsh_band
                    print "Start clustering for Band:", lsh_band
                    sys.stdout.flush()

                if prev_lsh_key != lsh_key:
                     if len(currentClusterInput) > 0:
                        if computeSimilarity:
                            self.__computeSimilarity(currentClusterInput,currentClusterBase,out,lsh_band)

                        else:
                            self.__writeClusters(currentClusterInput,currentClusterBase,out)


                     del currentClusterInput[:]
                     del currentClusterBase[:]

                if prev_lsh_band != lsh_band:
                    print "Start clustering for Band:", lsh_band
                    sys.stdout.flush()

                prev_lsh_key = lsh_key
                prev_lsh_band = lsh_band

                if basePrefix is not None and itemId.startswith(basePrefix):
                    currentClusterBase.append(lineTokens[1:])
                else:
                    currentClusterInput.append(lineTokens[1:])

        if len(currentClusterInput) > 0:
            if computeSimilarity:
                self.__computeSimilarity(currentClusterInput,currentClusterBase,out,lsh_band)

            else:
                self.__writeClusters(currentClusterInput,currentClusterBase,out)

        file.close()
        out.close()
        print "Done computing similarities"
        sys.stdout.flush()


    def __writeClusters(self, keyHashesInput, keyHashesBase, outputFile):
        keyArr = []
        for keyArr1 in keyHashesInput:
            key1 = keyArr1[0]
            keyArr.append(key1)
        for keyArr2 in keyHashesBase:
            key2 = keyArr2[0]
            keyArr.append(key2)

        outputFile.write(util.write_tokens(keyArr, self.separator) + "\n")

    def __computeSimilarity(self, keyHashesInput,keyHashesBase, outputFile, lsh_band):
        #print "Compute Similarity between: ", keys with inputprefix and keys with baseprefix, "
        numLines = 0
        for keyArr1 in keyHashesInput:
            key1 = keyArr1[0]
            minarr1 = keyArr1[1:]
            #print "Start: ", key1
            for keyArr2 in keyHashesBase:
                key2 = keyArr2[0]
                minarr2 = keyArr2[1:]
                if minarr1 != minarr2:
                    score = util.compute_list_similarity(minarr1, minarr2)
                else:
                    score = 1.0
                if score >= self.scorethreshold:
                    if score < 1.0 or lsh_band == "000":
                        outputFile.write(key1 + self.separator + key2 + self.separator + str(score) + "\n")
                        numLines += 1
        return numLines



    def setNumRowsPerFile(self, num):
        self.numRowsPerFile = num


    def remove_duplicates(self, filename):
        util.sort_csv_file(filename, [0, 1], self.separator)

        file_handle = open(filename, 'r')
        tmp_file = open(filename + ".tmp", 'w')
        prev_line = None
        for line in file_handle:
            if prev_line is None or line != prev_line:
                tmp_file.write(line)
            prev_line = line

        os.remove(filename)
        os.rename(filename + ".tmp", filename)


inputFilename = None
baseFilename = None
inputPrefix = None
basePrefix = None
outputFilename = None
separator = "\t"
numHashes = 20
numItemsInBand = 5
minItemsInCluster = 2
dataType = "integer"
computeSimilarity = True
scoreThreshold = 0.0
removeDuplicates = True


def parse_args():
    global inputFilename
    global inputPrefix
    global outputFilename
    global separator
    global numHashes
    global numItemsInBand
    global baseFilename
    global basePrefix
    global minItemsInCluster
    global dataType
    global scoreThreshold
    global computeSimilarity

    for arg_idx, arg in enumerate(sys.argv):
        if arg == "--input":
            inputFilename = sys.argv[arg_idx+1]
            continue
        if arg == "--inputPrefix":
            inputPrefix = sys.argv[arg_idx+1]
            continue
        if arg == "--base":
            baseFilename = sys.argv[arg_idx+1]
            continue
        if arg == "--basePrefix":
            basePrefix = sys.argv[arg_idx+1]
            continue
        if arg == "--output":
            outputFilename = sys.argv[arg_idx+1]
            continue
        if arg == "--separator":
            separator = sys.argv[arg_idx+1]
            continue
        if arg == "--numHashes":
            numHashes = int(sys.argv[arg_idx+1])
            continue
        if arg == "--numItemsInBand":
            numItemsInBand = int(sys.argv[arg_idx+1])
            continue
        if arg == "--minItemsInCluster":
            minItemsInCluster = int(sys.argv[arg_idx+1])
            continue
        if arg == "--dataType":
            dataType = sys.argv[arg_idx+1]
            continue
        if arg == "--threshold":
            scoreThreshold = float(sys.argv[arg_idx+1])
            continue
        if arg == "--computeSimilarity":
            computeSimilarityStr = (sys.argv[arg_idx+1])
            if computeSimilarityStr == "False":
                computeSimilarity = False
            continue

def die():
    print "Please input the required parameters"
    print "Usage: genClustersWithBase.py --input <input filename> --inputPrefix <input Prefix> --base <base filename> --basePrefix <base prefix> " \
          "--output <output dir> [--separator <sep=\\t>] [--numHashes <numHashes=20>] [--numItemsInBand <numItemsInBand=5>]" \
          " [--minItemsInCluster <minItemsInCluster=2>] [--dataType <default=integer|string>]" \
          " [--threshold <threshold for similarity score to be in one cluster>]"
    exit(1)

parse_args()
if inputFilename is None or outputFilename is None \
        or inputPrefix is None:
    die()

print "Generate Clusters: ", inputFilename,baseFilename,  ", computeSimilarity:", computeSimilarity, ", scoreThreshold:", scoreThreshold, ", removeDuplicates:", removeDuplicates
sys.stdout.flush()

clusterGen = ClusterWithBaseGenerator()

clusterGen.run(inputFilename,baseFilename,outputFilename,separator,computeSimilarity,scoreThreshold)


# this will remove all the duplicates
if computeSimilarity and removeDuplicates:
   clusterGen.remove_duplicates(outputFilename)




__author__ = 'dipsy'
import sys
import os
from hdfs.hfile import Hfile

sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from utils import util

class ClusterGenerator(object):

    def __init__(self):
        self.separator = "\t"
        self.score_threshold = 0.0

    def __open_file_for_write(self, file_handle):
        if file_handle.startswith("hdfs://"):
            path = file_handle[7:]
            idx = path.find(":")
            host = path[0:idx]
            path = path[idx+1:]
            idx = path.find("/")
            port = int(path[0:idx])
            filename = path[idx:]
            print "Open HDFS file: host:", host + ",port:", port, ", filename:", filename
            sys.stdout.flush()
            return Hfile(host, port, filename, mode='w')
        else:
            print "Open file: ", file_handle
            sys.stdout.flush()
            return open(file_handle, 'w')

    def run(self, input_filename, output_filename, p_separator, compute_similarity, p_score_threshold):
        self.separator = p_separator
        self.score_threshold = p_score_threshold

        key_hashes = []
        prev_lsh_key = None
        prev_lsh_band = None

        in_file = open(input_filename, 'r')

        out = self.__open_file_for_write(output_filename)
        for line in in_file:
            line = line.strip()
            if len(line) > 0:
                line_tokens = line.split(separator)
                lsh_key = line_tokens[0]
                lsh_band = lsh_key[0:3]

                key_hash = line_tokens[1:]

                if prev_lsh_key is None:
                    prev_lsh_key = lsh_key
                    prev_lsh_band = lsh_band
                    print "Start clustering for Band:", lsh_band
                    sys.stdout.flush()

                if prev_lsh_key != lsh_key:
                    if len(key_hashes) > 1:
                        if compute_similarity:
                            self.__compute_similarity(key_hashes, out, lsh_band)
                        else:
                            self.__write_clusters(key_hashes, out)
                    del key_hashes[:]

                if prev_lsh_band != lsh_band:
                    print "Start clustering for Band:", lsh_band
                    sys.stdout.flush()

                prev_lsh_key = lsh_key
                prev_lsh_band = lsh_band

                key_hashes.append(key_hash)

        if len(key_hashes) > 1:
            if compute_similarity:
                self.__compute_similarity(key_hashes, out, lsh_band)
            else:
                self.__write_clusters(key_hashes, out)
        in_file.close()
        out.close()
        print "Done computing similarities"
        sys.stdout.flush()

    def __write_clusters(self, key_hashes_array, output_file):
        key_arr = []
        for key_arr1 in key_hashes_array:
            key1 = key_arr1[0]
            key_arr.append(key1)

        output_file.write(util.write_tokens(key_arr, self.separator) + "\n")

    def __compute_similarity(self, key_hashes_array, output_file, lsh_band):
        #print "Compute Similarity between: ", len(keyHashesArray), " items"
        num_lines = 0
        for keyArr1 in key_hashes_array:
            key1 = keyArr1[0]
            min_arr1 = keyArr1[1:]
            #print "Start: ", key1
            for keyArr2 in key_hashes_array:
                key2 = keyArr2[0]
                if key1 < key2:
                    min_arr2 = keyArr2[1:]
                    if min_arr1 != min_arr2:
                        score = util.compute_list_similarity(min_arr1, min_arr2)
                    else:
                        score = 1.0
                    if score >= self.score_threshold:
                        if score < 1.0 or lsh_band == "000":
                            output_file.write(key1 + self.separator + key2 + self.separator + str(score) + "\n")
                            num_lines += 1
        return num_lines

    def remove_duplicates(self, filename):
        util.sort_csv_file(filename, [0, 1], self.separator)

        file_handle = open(filename, 'r')
        tmp_file = open(filename + ".tmp", 'w')
        prev_line = None
        for line in file_handle:
            if prev_line is None or line != prev_line:
                tmp_file.write(line)
            prev_line = line

        os.remove(filename)
        os.rename(filename + ".tmp", filename)

inputFilename = None
outputFilename = None
separator = "\t"
dataType = "integer"
computeSimilarity = True
scoreThreshold = 0.0
removeDuplicates = True


def parse_args():
    global inputFilename
    global outputFilename
    global separator
    global scoreThreshold
    global computeSimilarity
    global removeDuplicates

    for arg_idx, arg in enumerate(sys.argv):
        if arg == "--input":
            inputFilename = sys.argv[arg_idx+1]
            continue
        if arg == "--output":
            outputFilename = sys.argv[arg_idx+1]
            continue
        if arg == "--separator":
            separator = sys.argv[arg_idx+1]
            continue
        if arg == "--threshold":
            scoreThreshold = float(sys.argv[arg_idx+1])
            continue
        if arg == "--computeSimilarity":
            computeSimilarityStr = (sys.argv[arg_idx+1])
            if computeSimilarityStr == "False":
                computeSimilarity = False
            continue
        if arg == "--removeDuplicates":
            removeDuplicatesStr = (sys.argv[arg_idx+1])
            if removeDuplicatesStr == "False":
                removeDuplicates = False
            continue


def die():
    print "Please input the required parameters"
    print "Usage: genClusters.py --input <input filename> --output <output filename> [--separator <sep=\\t>] [--computeSimilarity <True=default|False>] [--threshold <threshold for similarity score to be in one cluster>]"
    exit(1)

parse_args()
if inputFilename is None or outputFilename is None:
    die()

print "Generate Clusters: ", inputFilename, ", computeSimilarity:", computeSimilarity, ", scoreThreshold:", scoreThreshold, ", removeDuplicates:", removeDuplicates
sys.stdout.flush()
clusterGen = ClusterGenerator()

clusterGen.run(inputFilename, outputFilename, separator, computeSimilarity, scoreThreshold)
if removeDuplicates:
    clusterGen.remove_duplicates(outputFilename)







__author__ = 'dipsy'

import sys
import os

sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from utils import util
from hasher.lsh import LSH, IntegerLSH, MinHashSignature, IntegerMinHashSignature

inputFilename = None
inputType = "tokens"
outputFilename = None
separator = "\t"
numHashes = 20
numItemsInBand = 5
dataType = "integer"
keyPrefix = ""
sortOutput = False
outputMinhash = False

def parse_args():
    global inputFilename
    global outputFilename
    global separator
    global numHashes
    global numItemsInBand
    global dataType
    global keyPrefix
    global inputType
    global sortOutput
    global outputMinhash

    for arg_idx, arg in enumerate(sys.argv):
        if arg == "--input":
            inputFilename = sys.argv[arg_idx+1]
            continue
        if arg == "--inputType":
            inputType = sys.argv[arg_idx+1]
            continue
        if arg == "--output":
            outputFilename = sys.argv[arg_idx+1]
            continue
        if arg == "--separator":
            separator = sys.argv[arg_idx+1]
            continue
        if arg == "--numHashes":
            numHashes = int(sys.argv[arg_idx+1])
            continue
        if arg == "--numItemsInBand":
            numItemsInBand = int(sys.argv[arg_idx+1])
            continue
        if arg == "--dataType":
            dataType = sys.argv[arg_idx+1]
            continue

        if arg == "--keyPrefix":
            keyPrefix = sys.argv[arg_idx+1]
            continue
        if arg == "--sortOutput":
            sortOutputStr = (sys.argv[arg_idx+1])
            if sortOutputStr == "True":
                sortOutput = True
            continue
        if arg == "--outputMinhash":
            outputMinhashStr = (sys.argv[arg_idx+1])
            if outputMinhashStr == "True":
                outputMinhash = True
            continue


def die():
    print "Please input the required parameters"
    print "Usage: genLSH.py --input <input filename> [--inputType <tokens|minhash>] --output <output filename> [--separator <sep=\\t>] " \
          "[--numHashes <numHashes=20>] [--numItemsInBand <numItemsInBand=5>] [--dataType <default=integer|string>]" \
          "[--keyPrefix <prefix for key] [--sortOutput <True|False=default] [--outputMinhash <True|False=default>"
    exit(1)

args = parse_args()
if inputFilename is None or outputFilename is None:
    die()

print "Generate LSh Keys: numHashes:", numHashes, ", numItemsInBand:", numItemsInBand, ", dataType:", dataType, ", sortOutput:", sortOutput, ", inputType:", inputType
hasher = None
signer = None
if dataType == "integer":
    hasher = IntegerLSH(numHashes, numItemsInBand, None)
else:
    hasher = LSH(numHashes, numItemsInBand, None)
if inputType == "tokens":
    if dataType == "integer":
        signer = IntegerMinHashSignature(numHashes)
    else:
        signer = MinHashSignature(numHashes)

file = open(inputFilename, 'r')
wFile = open(outputFilename, 'w')
numBands = numHashes/numItemsInBand

for line in file:
    idx = line.find("\t")
    if idx != -1:
        key = line[0:idx]
        line_len = len(line)
        data = line[idx + 1:line_len-1].strip() #remove the \n
        if len(data) > 0:
            #print "Data:" + data
            tokens = data.split(separator)
            if len(tokens) > 0:
                #print "Adding tokens: " + str(tokens)
                minHashSig = None
                if inputType == "tokens":
                    if dataType == "integer":
                        tokens = util.get_int_list(tokens)
                    #print "Sign", tokens
                    if len(tokens) > 0:
                        minHashSig = signer.sign(tokens)
                else:
                    minHashSig = tokens

                #print "Key: ", key + ", minHash", minHashSig
                if minHashSig is not None:
                    lshSig = list(hasher.hash(minHashSig))
                    minOut = ""
                    if outputMinhash:
                        minOut = separator + util.write_tokens(minHashSig, separator)

                    for i in range(0, numBands):
                        wFile.write(str(i).zfill(3) + ":" + lshSig[i] + separator + keyPrefix + key + minOut + "\n")

file.close()
wFile.close()

if sortOutput:
    print "Sorting output on LSH Keys.."
    util.sort_csv_file(outputFilename, [0], separator)



__author__ = 'rajagopal'

__all__ = ["hasher.hasher", "hasher.lsh.lsh",
           "cluster.clusterer", "clusterer.unionFind", "clusterer.computeClusterPrecisionRecall",
           "gen_int_input.mr_str_to_int_mapper","gen_int_input.str_to_int_tokens",
           "utils.address","utils.util"]
from hasher.hasher import *
from hasher.lsh.lsh import *
from clusterer.clusterer import *
from clusterer.unionFind import *
from utils.address import *
from utils.util import *



#!/usr/bin/env python

__author__ = 'dipsy'

import zipimport
importer = zipimport.zipimporter('python-packages.mod')
gensim = importer.load_module('gensim')

import sys
from gensim import corpora
import json
import os
import subprocess

key_dictionary = corpora.Dictionary()
token_dictionary = corpora.Dictionary()



def get_line_representation(self, line):
    global key_dictionary
    global token_dictionary
    all_tokens = line.lower().split('\t')
    key = all_tokens[0]
    tokens = all_tokens[1:]
    yield key_dictionary.token2id.get(key)
    for token in tokens:
        yield token_dictionary.token2id.get(token)


for line in sys.stdin:
    line = line.strip()
    idx = line.find("\t")
    if idx != -1:
        try:
            key = line[0:idx]
            tokens =  line[idx+1:].split('\t')
            key_dictionary.doc2bow([key], allow_update=True)
            token_dictionary.doc2bow(tokens, allow_update=True)

            print get_line_representation(line)
        except Exception as e:
            print >> sys.stderr, e
            pass


#output_dir = os.getenv('mapred_output_dir')
#idx = output_dir.rfind("/")
#output_dir = output_dir[0:idx]
#key_filename = output_dir + '/output-key-' + os.getenv('mapred_task_id')
#key_proc = subprocess.Popen(['hadoop', 'fs', '-put', '-', key_filename], stdin=subprocess.PIPE)
#key_dictionary.save(key_proc.stdin)
#key_proc.stdin.close()
#key_proc.wait()

#token_filename = output_dir + '/output-token-' + os.getenv('mapred_task_id')
#token_proc = subprocess.Popen(['hadoop', 'fs', '-put', '-', token_filename], stdin=subprocess.PIPE)
#token_dictionary.save(token_proc.stdin)
#token_proc.stdin.close()
#token_proc.wait()




#!/usr/bin/env python

import sys

current_id = None

# input comes from STDIN
for line in sys.stdin:
    # remove leading and trailing whitespace
    line = line.strip()

    # parse the input we got from mapper.py
    (cluster_id, size) = line.split('\t', 1)

    # this IF-switch only works because Hadoop sorts map output
    # by key (here: word) before it is passed to the reducer
    if current_id == cluster_id:
        continue
    else:
        print '%s\t%s' % (cluster_id, size)
        current_id = cluster_id

exit(0)



#!/usr/bin/env python

import sys

# input comes from STDIN (standard input)
for line in sys.stdin:
    # remove leading and trailing whitespace
    line = line.strip()
    idx = line.find("\t")
    if idx != -1:
        print line




#!/usr/bin/env python

import sys
import hashlib

# input comes from STDIN (standard input)
for line in sys.stdin:
    # remove leading and trailing whitespace
    line = line.strip()
    if len(line) > 0:
        (lsh, cluster_id, cluster_count, minhash) = line.split("\t", 3)
        print "%s\t%s" % (cluster_id, cluster_count)
exit(0)




#!/usr/bin/env python

import sys
import hashlib

# input comes from STDIN (standard input)
for line in sys.stdin:
    # remove leading and trailing whitespace
    line = line.strip()
    if len(line) > 0:
        print line

exit(0)




#!/usr/bin/env python
import re
import sys
import os
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

us_states_names = [
    "ALABAMA", "ALASKA", "ARIZONA", "ARKANSAS", "CALIFORNIA", "COLORADO", "CONNECTICUT", "DELAWARE", "FLORIDA",
    "GEORGIA", "HAWAII", "IDAHO", "ILLINOIS", "INDIANA", "IOWA", "KANSAS", "KENTUCKY", "LOUISIANA", "MAINE", "MARYLAND",
    "MASSACHUSETTS", "MICHIGAN", "MINNESOTA", "MISSISSIPPI", "MISSOURI", "MONTANA", "NEBRASKA", "NEVADA",
    "NEWHAMPSHIRE", "NEWJERSEY", "NEWMEXICO", "NEWYORK", "NORTHCAROLINA", "NORTHDAKOTA", "OHIO", "OKLAHOMA", "OREGON",
    "PENNSYLVANIA", "RHODEISLAND", "SOUTHCAROLINA", "SOUTHDAKOTA", "TENNESSEE", "TEXAS", "UTAH", "VERMONT", "VIRGINIA",
    "WASHINGTON", "WESTVIRGINIA", "WISCONSIN", "WYOMING"
]

us_states_names_readable = [
     "Alabama", "Alaska", "Arizona", "Arkansas", "California", "Colorado", "Connecticut", "Delaware", "Florida",
    "Georgia", "Hawaii", "Idaho", "Illinois", "Indiana", "Iowa", "Kansas", "Kentucky", "Louisiana", "Maine", "Maryland",
    "Massachusetts", "Michigan", "Minnesota", "Mississippi", "Missouri", "Montana", "Nebraska", "Nevada",
    "New Hampshire", "New Jersey", "New Mexico", "New York", "North Carolina", "North Dakota", "Ohio", "Oklahoma", "Oregon",
    "Pennsylvania", "Rhode Island", "South Carolina", "South Dakota", "Tennessee", "Texas", "Utah", "Vermont", "Virginia",
    "Washington", "West Virginia", "Wisconsin", "Wyoming"
]

us_states_codes = [
    "AL", "AK", "AZ", "AR", "CA", "CO", "CT", "DE", "FL", "GA", "HI", "ID", "IL", "IN", "IA", "KS", "KY", "LA", "ME",
    "MD", "MA", "MI", "MN", "MS", "MO", "MT", "NE", "NV", "NH", "NJ", "NM", "NY", "NC", "ND", "OH", "OK", "OR", "PA",
    "RI", "SC", "SD", "TN", "TX", "UT", "VT", "VA", "WA", "WV", "WI", "WY"
]

def alphaOnly(x):
    "Remove non-alphabetic chars from the string x"
    return re.sub('[^A-Za-z]+', '', x)

def standardize_state_code(state):
    if(len(state) == 0):
        return ''

    codesArr = []
    namesArr = []


    namesArr = us_states_names
    codesArr = us_states_codes


    if (len(codesArr) > 0):
        state = state.upper()
        if (len(state) == 2):
            try:
                idx = codesArr.index(state)
                return state
            except ValueError:
                idx = -1

        try:
            state = alphaOnly(state).upper()
            idx = namesArr.index(state)
            code = codesArr[idx]
            return code
        except ValueError:
            return ''

    return state


def standardize_state_name(state):
    if(len(state) == 0):
        return ''

    codesArr = us_states_codes
    namesArr = us_states_names
    namesReadableArr = us_states_names_readable

    if (len(codesArr) > 0):
        state = state.upper()
        if (len(state) == 2):
            try:
                idx = codesArr.index(state)
                return namesReadableArr[idx]
            except ValueError:
                idx = -1

        try:
            state = alphaOnly(state).upper()
            idx = namesArr.index(state)
            return namesReadableArr[idx]
        except ValueError:
            return ''

    return state



#!/usr/bin/env python
from pyspark import SparkContext
from optparse import OptionParser
import json
from digSparkUtil.fileUtil import FileUtil

# Implementation Of: https://chasebradford.wordpress.com/2010/10/23/mapreduce-implementation-for-union-find/

def prepareMap(tuple):
    key = tuple[0]
    value = tuple[1]

    if key != "OPEN":
        value.append(key)

    # print "PREPARE:", value

    return "OPEN", value


def electMap(tuple):
    x = tuple[1]

    pick = str(x[0])
    for z in x[1:]:
        z = str(z)
        if z < pick:
            pick = z
    res = []
    for z in x:
        z = str(z)
        if z != pick:
            # print "ELECTMAP:", z, ",", [pick]
            yield z, [pick]
            res.append(z)

    # print "ELECTMAP:", pick, ",", res
    yield pick, res


def finalMap(tuple):
    # print "FINAL MAP:", tuple
    pick = str(tuple[0])
    x = tuple[1]

    for z in x:
        z = str(z)
        if z < pick:
            pick = z
    res = []
    for z in x:
        z = str(z)
        if z != pick:
            res.append(z)

    # print "FINALMAP RETURNS:", pick, ",", res
    return (pick, res)


def electReduce(val1, val2):
    # print "electReduce: v1:", val1, "v2:", val2
    x = set(val1)
    for z in val2:
        x.add(z)
    # print "ELECTREDUCE:", list(x)
    return list(x)


def partitionMap(tuple):
    key = tuple[0]
    value = tuple[1]
    # print "Got key:", key
    if len(value) > 1:
        # print "PARTITIONMAP:", key, ",", value
        return key, value
    else:
        # print "PARTITIONMAP:", value[0], ",", [key]
        return value[0], [key]


def partitionReduce(val1, val2):
    global OPEN
    counts = dict()
    for val in val1:
        counts[str(val)] = 1

    for val in val2:
        if str(val) in counts:
            counts[str(val)] = counts[str(val)] + 1
        else:
            counts[str(val)] = 1

    openValue = False
    res = set()
    for val in val1:
        if counts[str(val)] == 1:
            openValue = True
        res.add(val)

    for val in val2:
        if counts[str(val)] == 1:
            openValue = True
        res.add(val)

    # print "PARTITIONREDUCE:", list(res)
    return list(res)


def partitionReduceOpen(val1, val2):
    # print "partitionReduceOpen:", val1, " :: ", val2
    val1.extend(val2)
    return val1


def compute_sum(rdd1, rdd2):
    # print "Compute_sum", rdd1[0], ":", rdd1[1], " AND ", rdd2[0], ":", rdd2[1]
    sum1 = 0
    if rdd1[1]:
        sum1 = check_if_open(rdd1[1])
    sum2 = 0
    if rdd2[1]:
        sum2 = check_if_open(rdd2[1])
    # print "Sum:", sum1, "+", sum2, "=", (sum1+sum2)
    return sum1 + sum2


def check_if_open(tuple):
    arr = tuple[1]
    counts = dict()
    for val in arr:
        if str(val) in counts:
            counts[str(val)] += 1
        else:
            counts[str(val)] = 1

    for val in arr:
        if counts[str(val)] != 2:
            return 1

    return 0


class UnionFind:
    def __init__(self, **options):
        self.sums = 0
        self.prevSums = -1
        self.numTries = 0
        self.options = options
        print 'in unionFind'



    def perform(self, rdd):
        rdd = self.read_input(rdd)
        while True:
            rdd = self.run(rdd)
            if self.isEnd():
                break
        rdd = self.format_output(rdd)
        return rdd

    def output_csv(self, key, matches, separator):
        line = str(key)
        for match in matches:
            if type(match) is list or type(match) is dict or type(match) is tuple:
                line = line + separator + str(match[0])
            else:
                line = line + separator + str(match)
        yield line

    def read_input(self, rdd):
        def parse_json(tuple):
            x = tuple[1]
            ##if it's in the form of rdd x will be a dictionary else it will be string
            if isinstance(x,dict):
                cluster = x["member"]
            else:
                cluster = json.loads(x)["member"]
            res = []
            for item in cluster:
                res.append(item["uri"])
            return res

        rdd = rdd.map(lambda x: ("OPEN", parse_json(x)))
        return rdd

    def format_output(self, rdd):
        rdd_map = rdd.map(finalMap)
        rdd_final = rdd_map.reduceByKey(electReduce)

        def save_as_json(tuple):
            key = tuple[0]
            json_obj = {"member": [],"a":"http://schema.dig.isi.edu/ontology/Cluster"}
            json_obj["member"].append({"uri": key,"a":"http://schema.org/WebPage"})
            #json_obj["uri"]=key
            concat_str =''
            for val in tuple[1]:
                json_obj["member"].append({"uri": val,"a":"http://schema.org/WebPage"})
                concat_str += val
            cluster_id = "http://dig.isi.edu/ht/data/" + str(hash(concat_str)% 982451653)
            json_obj["uri"]=cluster_id
            return cluster_id + "/cluster", json_obj

        rdd_final = rdd_final.map(save_as_json)
        return rdd_final

    def run(self, rdd):
        process_rdd = rdd.map(prepareMap)
        rdd_elect_map = process_rdd.flatMap(electMap)
        rdd_elect_reduce = rdd_elect_map.reduceByKey(electReduce)
        rdd_partition_map = rdd_elect_reduce.map(partitionMap)
        self.sums = rdd_partition_map.reduceByKey(partitionReduceOpen).map(check_if_open).sum()
        print "Got sums:", self.sums

        rdd_partition_reduce = rdd_partition_map.reduceByKey(partitionReduce)
        # print "RDD:", rdd_partition_reduce.collect()
        return rdd_partition_reduce

    def isEnd(self):
        # print "============================================================"
        end = False
        if self.sums == 0:
            end = True
        elif self.prevSums == self.sums:
            self.numTries += 1
            if self.numTries >= 4:
                end = True
        else:
            self.numTries = 0

        self.prevSums = self.sums

        return end

'''
if __name__ == "__main__":
    sc = SparkContext(appName="LSH-UNION")
    parser = OptionParser()
    parser.add_option("-r", "--separator", dest="separator", type="string",
                      help="field separator", default="\t")
    parser.add_option("-i", "--inputformat", dest="inputformat", type="string",
                      help="input file format: text/sequence", default="text")
    parser.add_option("-t", "--inputtype", dest="inputtype", type="string",
                      help="output type: csv/json", default="json")
    parser.add_option("-o", "--outputformat", dest="outputformat", type="string",
                      help="output file format: text/sequence", default="text")
    parser.add_option("-y", "--outputtype", dest="outputtype", type="string",
                      help="output type: csv/json", default="json")

    (c_options, args) = parser.parse_args()
    inputFilename = args[0]
    outputFilename = args[1]

    unionFind = UnionFind()
    rdd = unionFind.read_input(inputFilename, c_options)

    while True:
        rdd = unionFind.run(rdd)
        if unionFind.isEnd():
            break
    # x = rdd_elect_reduce.flatMap(lambda key, value: unionFind.output_csv(key, value, c_options.separator))
    unionFind.save_output(rdd, outputFilename, c_options)
'''


#!/usr/bin/env python

from pyspark import SparkContext
import sys



if __name__ == "__main__":
    def extractNumLines(line):
        global lines
        lines += 1
        # print lines, ":", line, "\n\n"
        return line

    sc = SparkContext(appName="CountKeys")
    file = sc.sequenceFile( sys.argv[1])
    rdd = file.reduceByKey(lambda x, y:  x)

    lines = sc.accumulator( 0)
    num_lines = rdd.map(extractNumLines)
    num_lines.collect()
    print "Num lines: %d" % lines.value



__author__ = 'dipsy'
from gensim import corpora
import sys
import subprocess
import os

class Corpus(object):
    def __init__(self, filename, filesystem, separator, convertKeys):
        self.filename = filename
        self.filesystem = filesystem
        self.separator = separator
        if convertKeys:
            self.key_dictionary = corpora.Dictionary()
        else:
            self.key_dictionary = None
        self.token_dictionary = corpora.Dictionary()
        print "Created blank token dictionary"

    def __iter__(self):
        print "Start the conversion"
        filenames = self.filename.split(",")
        for filename in filenames:
            print "File: ", filename
            stream_arr = []
            if inputFileSystem == "hdfs":
                cat = subprocess.Popen(["hadoop", "fs", "-cat", filename], stdout=subprocess.PIPE)
                stream = cat.stdout
                stream_arr.append(stream)
            else:
                if os.path.isdir(filename):
                    print "This is a directory, get all files"
                    filename_arr = os.listdir(filename)
                    for inner_filename in filename_arr:
                        full_name = os.path.join(filename, inner_filename)
                        stream = open(full_name)
                        stream_arr.append(stream)
                else:
                    print "INput is a file"
                    stream = open(filename)
                    stream_arr.append(stream)

            for stream in stream_arr:
                print "Got stream:", stream
                for line in stream:
                    line = line.decode("utf-8")
                    idx = line.find(self.separator)
                    if idx != -1:
                        key = line[0:idx].strip()
                        tokens = line[idx+1:].strip().lower().split(self.separator)
                        if self.key_dictionary:
                            self.key_dictionary.doc2bow([key], allow_update=True)
                        self.token_dictionary.doc2bow(tokens, allow_update=True)
                        #print line
                        yield self.get_line_representation(key, tokens)

    def get_key_hashmap(self):
        return (self.key_dictionary.token2id)

    def get_token_hashmap(self):
        return self.token_dictionary.token2id

    def save_key_dictionary(self, filename):
        self.key_dictionary.save(filename)
        self.key_dictionary.save_as_text(filename + ".txt")

    def load_key_dictionary(self, filename):
        self.key_dictionary.load(filename)

    def save_token_dictionary(self, filename):
        self.token_dictionary.save(filename)
        self.token_dictionary.save_as_text(filename + ".txt")
        print self.token_dictionary

    def load_token_dictionary(self, filename):
        print "Load the token dictionary from file: " + filename
        self.token_dictionary = corpora.Dictionary.load(filename)
        print self.token_dictionary

    def get_line_representation(self, key, tokens):
        if self.key_dictionary:
            yield self.get_key_hashmap().get(key)
        else:
            yield key
        for token in tokens:
            yield self.get_token_hashmap().get(token)


inputFile = None
inputFileSystem = "file"
outputFile = None
keyHashFile = None
tokenHashFile = None
separator = "\t"
inputTokensHashFile = None

def parse_args():
    global inputFile
    global inputFileSystem
    global outputFile
    global keyHashFile
    global tokenHashFile
    global separator
    global inputTokensHashFile

    for arg_idx, arg in enumerate(sys.argv):
        if arg == "--input":
            inputFile = sys.argv[arg_idx+1]
            continue
        if arg == "--output":
            outputFile = sys.argv[arg_idx+1]
            continue
        if arg == "--inputFS":
            inputFileSystem = sys.argv[arg_idx+1]
            continue
        if arg == "--outputKeysFile":
            keyHashFile = (sys.argv[arg_idx+1])
            continue
        if arg == "--outputTokensFile":
            tokenHashFile = (sys.argv[arg_idx+1])
            continue
        if arg == "--separator":
            separator = sys.argv[arg_idx+1]
            continue
        if arg == "--inputTokensFile":
            inputTokensHashFile = sys.argv[arg_idx+1]
            continue

def die():
    print "Please input the required parameters"
    print "Usage: str_to_int_tokens.py --input <input filename> [--inputFS <hdfs|default=file>] " \
          "--output <output filename> --outputKeysFile <keys map filename> --outputTokensFile <tokens map filename>" \
          "--separator <separator-default=tab> [--inputTokensFile <Tokens Hashfile to use as input>]"
    exit(1)

def write_tokens(tokens, sep):
    outStr = ""
    usep = ""
    for token in tokens:
        outStr = outStr.encode("ascii",'ignore') + usep.encode("ascii","ignore") + str(token if isinstance(token,int) else token.encode("ascii","ignore"))
        usep = sep
    return outStr

args = parse_args()
if inputFile is None or outputFile is None or tokenHashFile is None:
    die()

try:
    outFP = open(outputFile, "w")
    corpus = Corpus(inputFile, inputFileSystem, separator, keyHashFile is not None)
    if inputTokensHashFile is not None:
        corpus.load_token_dictionary(inputTokensHashFile)

    for vectors in corpus:
        vector_list = list(vectors)
        outFP.write(write_tokens(vector_list, separator) + "\n")

    outFP.close()

    if keyHashFile is not None:
        corpus.save_key_dictionary(keyHashFile)
    corpus.save_token_dictionary(tokenHashFile)
    print "Done conversion"
except Exception as e:
    print "Exception:" + e.message



#!/usr/bin/env python

import sys


current_key = None
cluster_ids = set()
word = None


def print_output(key, cluster_id_arr):
    for cluster_id_val in cluster_id_arr:
        print "%s\t%s" % (key, cluster_id_val)


for line in sys.stdin:
    # remove leading and trailing whitespace
    line = line.strip()

    (key, cluster_id) = line.split('\t', 1)

    # this IF-switch only works because HADOOP sorts map output
    # by key (here: word) before it is passed to the reducer
    if current_key == key:
        cluster_ids.add(cluster_id)
    else:
        if current_key:
            if len(cluster_ids) > 0:
                print_output(current_key, cluster_ids)
            cluster_ids.clear()

        current_key = key
        cluster_ids.add(cluster_id)

if len(cluster_ids) > 0:
    print_output(current_key, cluster_ids)

exit(0)



#!/usr/bin/env python

import sys

current_lsh = None
key_hashes = []
word = None



def compute_list_similarity(list1, list2):
    #similarity = float(len(set(list1) & set(list2)))/float(len(set(list1)))
    list_len = len(list1)
    num_match = 0
    for i in range(0, list_len):
        if list1[i] == list2[i]:
            num_match += 1
    similarity = float(num_match)/float(list_len)
    return similarity


def print_similarity(key_hash_strings, lsh_key):
    for keyArr1 in key_hash_strings:
        (key1, count1, hash_str1) = keyArr1.split("\t", 2)
        hash1 = hash_str1.split("\t")
        for keyArr2 in key_hash_strings:
            (key2, count2, hash_str2) = keyArr2.split("\t", 2)
            hash2 = hash_str2.split("\t")
            if key1 < key2:
                if hash_str1 != hash_str2:
                    score = compute_list_similarity(hash1, hash2)
                else:
                    score = 1.0
                lsh_band = lsh_key[0:3]
                if score < 1.0 or lsh_band == "000":
                    print '%s\t%s\t%s\t%s\t%s' % (key1, key2, str(score),
                                                  str(count1),
                                                  str(count2)
                                                )

for line in sys.stdin:
    # remove leading and trailing whitespace
    line = line.strip()

    (lsh, cluster_id, count, minhash) = line.split('\t', 3)

    # this IF-switch only works because HADOOP sorts map output
    # by key (here: word) before it is passed to the reducer
    if current_lsh == lsh:
        key_hashes.append(cluster_id + "\t" + count + "\t" + minhash)
    else:
        if current_lsh:
            if len(key_hashes) > 1:
                print_similarity(key_hashes, current_lsh)
            del key_hashes[:]
        current_lsh = lsh
        key_hashes.append(cluster_id + "\t" + count + "\t" + minhash)

if len(key_hashes) > 1:
    print_similarity(key_hashes, current_lsh)

exit(0)





#!/usr/bin/env python

from pyspark import SparkContext
import sys



if __name__ == "__main__":
    sc = SparkContext(appName="CopyText")
    in_file = sc.textFile( sys.argv[1])
    in_file.saveAsTextFile(sys.argv[2])



__author__ = 'rajagopal067'
import sys
import os

sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from utils import util
import fileinput

similarityList=[]
def findSimilaritySingleFile(LSHKey,keyHashesArray):
    i=0
    while i< len(keyHashesArray):
        j=i+1
        while j < len(keyHashesArray):
            findSimilarityUtil(LSHKey,keyHashesArray[i],keyHashesArray[j])
            j=j+1
        i=i+1

def findSimilarity(LSHKey,keysInput,keysBase):
    if len(keysBase) == 0:
        findSimilaritySingleFile(LSHKey,keysInput)
    else:
        i=0
        while i<len(keysInput):
            j=0
            while j<len(keysBase):
                findSimilarityUtil(LSHKey,keysInput[i],keysBase[j])
                j=j+1
            i=i+1

def findSimilarityUtil(LSHKey,list1,list2):
    key1 = list1[0]
    key2=list2[0]

    hashlist1 = []
    hashlist2 = []
    hashlist1 = list1[1:len(list1)]
    hashlist2 = list2[1 : len(list2)]

    similarity = float(len(set(hashlist1) & set(hashlist2)))/float(len(set(hashlist1)))
    tmpList=[LSHKey,key1,key2,similarity]

    similarityList.append(tmpList)

def write_ToFile():
    outFP = open(outputFilename,"w")
    i=0
    similaritySet=set(map(tuple,similarityList))
    similarityList1=map(list,similaritySet)

    while i<len(similarityList1):
        list1=similarityList1[i]
        string = list1[0] + separator + list1[1] + separator + list1[2] + separator + str(list1[3])+"\n"
        outFP.write(string)
        i=i+1

inputFilename = None
baseFilename = None
inputPrefix = None
basePrefix = None
outputFilename = None
separator = "\t"
dataType = "integer"


def parse_args():
    global inputFilename
    global outputFilename
    global separator

    for arg_idx, arg in enumerate(sys.argv):
        if arg == "--input":
            inputFilename = sys.argv[arg_idx+1]
            continue
        if arg == "--inputPrefix":
            inputPrefix = sys.argv[arg_idx+1]
            continue
        if arg == "--base":
            baseFilename = sys.argv[arg_idx+1]
            continue
        if arg == "--basePrefix":
            basePrefix = sys.argv[arg_idx+1]
            continue
        if arg == "--output":
            outputFilename = sys.argv[arg_idx+1]
            continue
        if arg == "--separator":
            separator = sys.argv[arg_idx+1]
            continue

def die():
    print "Please input the required parameters"
    print "Usage: findSimilarity.py --input <input filename> [--inputPrefix <prefix value> --base <base filename> --basePrefix <prefix value>]--output <output filename> [--separator <sep=\\t>] "
    exit(1)

args = parse_args()
if inputFilename is None or outputFilename is None:
    die()

if baseFilename is not None:
    file_list = [inputFilename,baseFilename]
    tmp_combined_file =  inputFilename + ".tmp"
    with open(tmp_combined_file, 'w') as file:
        input_lines = fileinput.input(file_list)
        file.writelines(input_lines)
    util.sort_csv_file(tmp_combined_file, [0], '\t')
    file=open(tmp_combined_file,'r')
else:
    file=open(inputFilename,'r')

previousLSHKey = None
lshKey = None
keysInput=[]
keysBase=[]


for line in file:
    lineParts = line.strip().split("\t")
    lshKey=lineParts[0]
    itemId=lineParts[1]
    idx = line.find("\t")
    line_len = len(line)
    data = line[idx +1 : line_len -1].strip()
    if len(data) > 0:
        tokens = data.split(separator)

    if lshKey != previousLSHKey and previousLSHKey is not None:
        findSimilarity(previousLSHKey,keysInput,keysBase)
        del keysInput[:]
        del keysBase[:]

    if len(tokens) > 0:
        if basePrefix is not None and itemId.startswith(basePrefix):
            keysBase.append(tokens)
        else:
            keysInput.append(tokens)
    previousLSHKey=lshKey

file.close()
findSimilarity(previousLSHKey,keysInput,keysBase)
write_ToFile()




#!/usr/bin/env python

import sys
import codecs
import json


def parse_args():
    global inputFilename
    global outputFilename
    global separator
    global headerStartIndex
    global dataStartIndex
    global groupByColumn
    global header
    global pretty_print

    for arg_idx, arg in enumerate(sys.argv):
        if arg == "--help":
            usage()
            exit(1)
        if arg == "--input":
            inputFilename = sys.argv[arg_idx+1]
            continue
        if arg == "--output":
            outputFilename = sys.argv[arg_idx+1]
            continue
        if arg == "--separator":
            separator = sys.argv[arg_idx+1]
            continue
        if arg == "--headerStartIndex":
            headerStartIndex = int(sys.argv[arg_idx+1])
            continue
        if arg == "--dataStartIndex":
            dataStartIndex = int(sys.argv[arg_idx+1])
            continue
        if arg == "--groupByColumn":
            groupByColumn = int(sys.argv[arg_idx+1])
            continue
        if arg == "--header":
            header = sys.argv[arg_idx+1].split(",")
            continue
        if arg == "--prettyPrint":
            if sys.argv[arg_idx+1].lower() == "true":
                pretty_print = True


def die():
    print "Please input the required parameters"
    usage()
    exit(1)


def usage():
    print "\nUsage: csvtojson.py --input <input filename> --output <output filename> [--separator <sep=\\t>] " \
          "[--headerStartIndex 1] [--dataStartIndex 2] [--groupByColumn 0] [--header <comma separated header names>] " \
          "[--prettyPrint True|False, default=False]"
    print "Note: All indices are 1 based, 0 means do not use"
    print "Note: If groupByColumn is used, the input should be sorted by the grouped column. The utility only groups" \
          " consecutively similar values\n"


def generate_empty_header(row):
    global separator
    header_arr = []
    index = 1
    for part in row.split(separator):
        header_arr.append("Column" + str(index))
        index += 1
    return header_arr


def parse_header(row):
    global separator
    header_arr = []
    for part in row.split(separator):
        header_arr.append(part)
    return header_arr


def generate_json(headings, row):
    global separator
    global groupByColumn

    index = 0
    row_json = {}
    row_parts = row.split(separator)

    if groupByColumn != 0:
        heading = headings[groupByColumn-1]
        row_json[heading] = row_parts[groupByColumn-1]
        row_json["matches"] = []
        part_json = {}
        row_json["matches"].append(part_json)

    for part in row_parts:
        heading = headings[index]
        if groupByColumn == 0:
            row_json[heading] = part
        else:
            if index != groupByColumn-1:
                part_json = row_json["matches"][0]
                part_json[heading] = part

        index += 1

    return row_json


def group_json_by_column(group, row, column_name):
    if group is None:
        return row, None
    if group[column_name] == row[column_name]:
        group["matches"].extend(row["matches"])
        return group, None
    else:
        return row, group

inputFilename = None
dataStartIndex = 2
headerStartIndex = 1
separator = "\t"
outputFilename = None
groupByColumn = 0
header = None
pretty_print = False

parse_args()
if inputFilename is None or outputFilename is None:
    die()

inputFile = codecs.open(inputFilename,'r',"utf-8")
outputFile = codecs.open(outputFilename,'w','utf-8')

lineNumber = 1
outputFile.write("[")
sep = ""
group_json = None
prev_line_json = None

for line in inputFile:
    line = line.strip()
    if header is None and lineNumber == headerStartIndex:
        header = parse_header(line)

    if lineNumber >= dataStartIndex:
        if header is None:
            header = generate_empty_header(line)
        line_json = generate_json(header, line)

        if groupByColumn == 0:
            outputFile.write(sep + "\n" + json.dumps(line_json))
            sep = ","
        else:
            group_json, line_json = group_json_by_column(group_json, line_json, header[groupByColumn-1])
            if line_json is not None:
                compact_json = {"source":line_json[header[groupByColumn-1]],"candidates":line_json["matches"]}
                if pretty_print is True:
                    outputFile.write(sep + "\n" + json.dumps(compact_json, indent=2, separators=(',', ': ')))
                else:
                    outputFile.write(sep + "\n" + json.dumps(compact_json))
                sep = ","



outputFile.write("\n]")
outputFile.close()
inputFile.close()



"""
lsh.py

Algorithms based on 'Mining of Massive Datasets'
"""

from collections import defaultdict
import hashlib
import sys
import traceback
from unionfind import UnionFind

class Signature(object):
    """Signature Base class."""

    def __init__(self, dim):
        self.dim = dim
        self.hashes = self.hash_functions()

    def hash_functions(self):
        """Returns dim different hash functions"""
        pass

    def sign(self, object):
        """Return the signature for object s"""
        pass


class MinHashSignature(Signature):
    """Creates signatures for sets/tuples using minhash."""

    def hash_functions(self):
        """Return dim different hash functions"""
        def hash_factory(n):
            return lambda x: self.hash(x, n)
             #return lambda x: hash("salt" + unicode(n) + unicode(x) + "salt")

        return [ hash_factory(_) for _ in range(self.dim) ]

    def hash(self, x, n):
        str_n = str(n)
        try:
            #print x
            combined = str_n + x.encode('utf-8', 'ignore')
            return hashlib.md5(combined).hexdigest()
        except:
            print "x:", type(x)
            print "n:", type(n)
            exc_type, exc_value, exc_traceback = sys.exc_info()
            lines = traceback.format_exception(exc_type, exc_value, exc_traceback)
            sys.stderr.write("\nError in hashing:" + str(lines))
            pass

    def sign(self, s):
        """Returns minhash signature for set s"""
        sig = [ float("inf") ] * self.dim
        for hash_ix, hash_fn in enumerate(self.hashes):
            sig[hash_ix] = min(hash_fn(value) for value in s)
            #sig[hash_ix] = min(self.callHashFunc(hash_fn, value) for value in s)
        return sig

    def callHashFunc(self, func, value):
        value = func(value)
        print "Called fun:" + str(func) + " --> " + str(value)

class IntegerMinHashSignature(MinHashSignature):
    """Creates signatures for sets/tuples using minhash."""
    #100 random ints generated using: numpy.random.randint(1, 1000, 100)

    # random_ints = [507, 196,  79, 522, 738, 996, 115, 343, 727, 872, 102, 447, 547,
    #    700, 753, 503, 120, 171, 476, 151, 379, 719, 124, 944, 875, 145,
    #    376, 260, 526, 116, 708, 534, 609, 694,  17, 929, 291,  73, 397,
    #    418, 259, 633, 755,  83,  26, 491,  90,  45,  77, 265, 159, 490,
    #    829, 229,  25, 316, 245, 980,  71, 917, 395, 927,  79, 109, 979,
    #      8, 753, 251, 517, 941, 737, 971, 989,  35, 854, 892, 637,  87,
    #    476, 361, 400, 588, 506,  47, 494, 550, 968, 870, 972,  75, 608,
    #    820,  77,  22, 870, 327, 384, 271, 896, 562]

    #numpy.random.randint(1000000000, 100000000000, 100)
    random_ints = [82949893771,  5419778746, 46469569449, 22970946667, 64913052057,
       30846103421, 67260756538, 40182107096, 11752165040, 74366005163,
       85216970102,  2272823491, 59023542528, 54301192507, 73869314336,
       35904756288, 61611427379, 80564592970, 12563742922, 28143806691,
       48294309034, 30049847753, 63105746317, 12943975253, 28714472376,
        8522932562, 95457682799, 73228048499, 50683270217, 48339137748,
       28789991874, 34401670180, 32565556877, 11118511944, 35342053125,
       51649521745, 64702577820, 96976616864, 16849054224, 29659340052,
       49484699975,  7078499804, 98375094643, 83933384411, 18710519956,
       33507617434, 49435388342, 94237554795, 41031191754, 16598040523,
       24887889270, 27494830443, 81198196893, 23107337705, 25007523289,
       21889710108,  9891520687, 37285339859, 74862978744, 55181668070,
        8663096209, 71643805429, 99916046749, 48209388432,  7344855414,
        9411373086, 35525522592, 60381079899, 58236694301, 50562071249,
       23989742009, 44245865115, 79716819073,  2571813437, 21101408153,
       75856524634,  9498767679, 59688557755, 64639873124, 75518784994,
       43686764642, 19964845890, 23990612083, 45201091985, 43786770136,
       21353607749,  4916237372, 35393718151, 36182999232, 97903389738,
       86229940156, 72982562932, 16175429494, 96402675290, 31865759910,
       89020080573, 40311482488,  4119598498, 50384630985, 63071589293]



    #def __init__(self, dim):
    #    self.random_ints = numpy.random.randint(1, 100*dim, dim)
    #    MinHashSignature.__init__(self, dim)

    def hash_functions(self):
        """Return dim different hash functions"""
        def hash_factory(n, random_ints):
            return lambda x: (((random_ints[n] + 1) * (x+1)) + 1) % 982451653
            #return lambda x: (((2*n + 1)*x) + 1) % 982451653

        return [ hash_factory(_, self.random_ints) for _ in range(self.dim) ]


class LSH(object):
    """Locality sensitive hashing.  Uses a banding approach to hash
    similar signatures to the same buckets."""
    def __init__(self, n, r, t):
        self.n = n
        self.r = r
        self.b = int(self.n / self.r)
        if t:
            self.t = t
        else:
            self.t = self.get_threshold()
        #print "Setting threshold=" + str(self.t)

    def hash(self, sig):
        """Generate hashvals for this signature"""
        for band in zip(*(iter(sig),) * self.r):
            #band = r values from sig at a time
            yield ''.join(list(band))
            #yield hashlib.sha1("salt" + unicode(band) + "tlas").hexdigest()
            #yield hash("salt" + unicode(band) + "tlas")


    def get_threshold(self):
        r = self.r
        b = self.b
        return (1. / b) ** (1. / r)

    def get_n_bands(self):
        return self.b

class IntegerLSH(LSH):
    def hash(self, sig):
        """Generate hashvals for this signature"""
        for band in zip(*(iter(sig),) * self.r):
            #band = r values from sig at a time
            yield ''.join(str(x).zfill(10) for x in band)

class Cluster(object):
    """Clusters sets with Jaccard similarity above threshold with high
    probability.

    Algorithm based on Rajaraman, "Mining of Massive Datasets":
    1. Generate set signature
    2. Use LSH to map similar signatures to same buckets
    3. Use UnionFind to merge buckets containing same values
    """
    def __init__(self, minHashLen=13, numRowsInBucket=2, threshold=None):
        self.unionfind = UnionFind()
        self.signer = MinHashSignature(minHashLen)
        self.hasher = LSH(minHashLen, numRowsInBucket, threshold)
        self.hashmaps = [defaultdict(list)
                         for _ in range(self.hasher.get_n_bands())]
        self.lshmap = {}

    def add_set(self, s, label=None):
        # A label for this set
        if not label:
            label = s

        # Add to unionfind structure
        self.unionfind[label]

        # Get signature
        sig = self.signer.sign(s)

        # Union labels with same LSH key in same band
        lshKeys = self.hasher.hash(sig)
        self.lshmap[label] = []

        for band_idx, hshval in enumerate(lshKeys):
            #print "Got band_idx, hashval: " + str(band_idx) + "," + str(hshval)
            self.hashmaps[band_idx][hshval].append(label)
            self.unionfind.union(label, self.hashmaps[band_idx][hshval][0])
            self.lshmap[label].append(hshval)

    def get_clusters(self, min_cluster_len):
        for band_idx in range(0,len(self.hashmaps)):
            #print "clusters>Got band_idx: " + str(band_idx)
            hashmap = self.hashmaps[band_idx]
            for key in hashmap:
                list = hashmap[key]
                if(len(list) > min_cluster_len):
                    yield list

    def get_clusters_with_hashes(self, min_cluster_len):
        for band_idx in range(0,len(self.hashmaps)):
            hashmap = self.hashmaps[band_idx]
            for key in hashmap:
                list = hashmap[key]
                if(len(list) > min_cluster_len):
                    list2 = []
                    for label in list:
                        if self.lshmap[label]:
                            list2.append((label, self.lshmap[label]))
                        else:
                            list2.append(label)
                    yield list2

    def get_cluster_unions(self, min_cluster_len):
        x = self.unionfind.sets()
        for set in x:
            if len(set) > min_cluster_len:
                yield set

    def get_min_hash(self, object):
        return list(self.signer.sign(object))

    def get_lsh_hash(self, object):
        sig = self.signer.sign(object)
        return list(self.hasher.hash(sig))


class IntegerCluster(Cluster):
    def __init__(self, minHashLen=13, numRowsInBucket=2, threshold=None):
        self.unionfind = UnionFind()
        self.signer = IntegerMinHashSignature(minHashLen)
        self.hasher = IntegerLSH(minHashLen, numRowsInBucket, threshold)
        self.hashmaps = [defaultdict(list)
                         for _ in range(self.hasher.get_n_bands())]
        self.lshmap = {}

    def add_set(self, s, label=None):
        # A label for this set
        if not label:
            label = s

        int_s = []
        for t in s:
            t = t.strip()
            if len(t) > 0:
                int_s.append(int(t))
        if len(int_s) > 0:
            Cluster.add_set(self, int_s, label)

def shingle(s, k):
    """Generate k-length shingles of string s"""
    k = min(len(s), k)
    for i in range(len(s) - k + 1):
        yield s[i:i+k]


def hshingle(s, k):
    """Generate k-length shingles then hash"""
    for s in shingle(s, k):
        yield hash(s)


def jaccard_sim(X, Y):
    """Jaccard similarity between two sets"""
    x = set(X)
    y = set(Y)
    return float(len(x & y)) / len(x | y)


def jaccard_dist(X, Y):
    """Jaccard distance between two sets"""
    return 1 - jaccard_sim(X, Y)





__author__ = 'dipsy'

import sys
import os

sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from utils import util
from hasher.lsh import MinHashSignature, IntegerMinHashSignature

inputFilename = None
outputFilename = None
separator = "\t"
numHashes = 20
dataType = "integer"

def parse_args():
    global inputFilename
    global outputFilename
    global separator
    global numHashes
    global dataType

    for arg_idx, arg in enumerate(sys.argv):
        if arg == "--input":
            inputFilename = sys.argv[arg_idx+1]
            continue
        if arg == "--output":
            outputFilename = sys.argv[arg_idx+1]
            continue
        if arg == "--separator":
            separator = sys.argv[arg_idx+1]
            continue
        if arg == "--numHashes":
            numHashes = int(sys.argv[arg_idx+1])
            continue
        if arg == "--dataType":
            dataType = sys.argv[arg_idx+1]
            continue

def die():
    print "Please input the required parameters"
    print "Usage: genMinhash.py --input <input filename> --output <output filename> [--separator <sep=\\t>] " \
          "[--numHashes <numHashes=20>] [--dataType <default=integer|string>]"
    exit(1)

args = parse_args()
if inputFilename is None or outputFilename is None:
    die()

print "dataType:" + dataType + ", numHashes=" + str(numHashes)

signer = None
if dataType == "string":
    signer = MinHashSignature(numHashes)
else:
    signer = IntegerMinHashSignature(numHashes)

file = open(inputFilename, 'r')
wFile = open(outputFilename, "w")
for line in file:
    idx = line.find("\t")
    if idx != -1:
        key = line[0:idx]
        line_len = len(line)
        data = line[idx + 1:line_len-1].strip() #remove the \n
        if len(data) > 0:
            #print "Data:" + data
            tokens = data.split(separator)
            if dataType == "integer":
                tokens = util.get_int_list(tokens)
            if len(tokens) > 0:
                #print "Adding tokens: " + str(tokens)
                sig = signer.sign(tokens)
                wFile.write(key + separator + util.write_tokens(sig, separator) + "\n")
file.close()
wFile.close()




#!/usr/bin/env python

import sys


def compute_list_similarity(list1, list2):
    similarity = float(len(set(list1) & set(list2)))/float(len(set(list1)))
    return similarity


def print_similarity(key_hashes, lsh_key):
    for keyArr1 in key_hashes:
        (key1, hash_str1) = keyArr1.split("\t", 1)
        hash_arr1 = hash_str1.split("\t")
        for keyArr2 in key_hashes:
            (key2, hash_str2) = keyArr2.split("\t", 1)
            if key1 < key2:
                hash_arr2 = hash_str2.split("\t")
                if hash_arr1 != hash_arr2:
                    score = compute_list_similarity(hash_arr1, hash_arr2)
                else:
                    score = 1.0
                lsh_band = lsh_key[0:3]
                if score < 1.0 or lsh_band == "000":
                    print '%s\t%s\t%s' % (key1, key2, str(score))

current_lsh = None
itemKey_hashes = []
word = None


for line in sys.stdin:
    # remove leading and trailing whitespace
    line = line.strip()

    (lsh, key_hash) = line.split('\t', 1)

    # this IF-switch only works because HADOOP sorts map output
    # by key (here: word) before it is passed to the reducer
    if current_lsh == lsh:
        itemKey_hashes.append(key_hash)
    else:
        if current_lsh:
            if len(itemKey_hashes) > 1:
                print_similarity(itemKey_hashes, current_lsh)
            del itemKey_hashes[:]
        current_lsh = lsh
        itemKey_hashes.append(key_hash)

if len(itemKey_hashes) > 1:
    print_similarity(itemKey_hashes, current_lsh)

exit(0)





#!/usr/bin/env python

import sys
import json

for line in sys.stdin:
    # remove leading and trailing whitespace
    line = line.strip()

    if len(line) > 0:
        # (key1, key2, score) = line.split('\t', 2)
        # jsonObj = json.loads("{\"Column_1\":\"" + key1 + "\", \"Column_2\":\"" +
        #                   key2 + "\", \"Column_3\":\"" + score + "\"}")
        # print "%s\t%s" % (key1, json.dumps(jsonObj))

        columns = line.split("\t")
        jsonStr = "{"
        sep = ""
        for i in range(0, len(columns)):
            column = columns[i]
            jsonStr += sep + "\"Column_" + str(i+1) + "\":\"" + column + "\""
            sep = ", "
        jsonStr += "}"
        jsonObj = json.loads(jsonStr)
        print "%s\t%s" % (columns[0], json.dumps(jsonObj))

exit(0)






"""
UnionFind.py

Source: http://www.ics.uci.edu/~eppstein/PADS/UnionFind.py

Union-find data structure. Based on Josiah Carlson's code,
http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/215912
with significant additional changes by D. Eppstein.
"""

from collections import defaultdict


class UnionFind(object):
    """Union-find data structure.

    Each unionFind instance X maintains a family of disjoint sets of
    hashable objects, supporting the following two methods:

    - X[item] returns a name for the set containing the given item.
      Each set is named by an arbitrarily-chosen one of its members; as
      long as the set remains unchanged it will keep the same name. If
      the item is not yet part of a set in X, a new singleton set is
      created for it.

    - X.union(item1, item2, ...) merges the sets containing each item
      into a single larger set.  If any item is not yet part of a set
      in X, it is added to X as one of the members of the merged set.
    """

    def __init__(self):
        """Create a new empty union-find structure."""
        self.weights = {}
        self.parents = {}

    def __getitem__(self, object):
        """Find and return the name of the set containing the object."""
        # check for previously unknown object
        if object not in self.parents:
            self.parents[object] = object
            self.weights[object] = 1
            return object

        # find path of objects leading to the root
        path = [object]
        root = self.parents[object]
        while root != path[-1]:
            path.append(root)
            root = self.parents[root]

        # compress the path and return
        for ancestor in path:
            self.parents[ancestor] = root
        return root

    def __iter__(self):
        """Iterate through all items ever found or unioned by this structure."""
        return iter(self.parents)

    def union(self, *objects):
        """Find the sets containing the objects and merge them all."""
        roots = [self[x] for x in objects]
        heaviest = max([(self.weights[r],r) for r in roots])[1]
        for r in roots:
            if r != heaviest:
                self.weights[heaviest] += self.weights[r]
                self.parents[r] = heaviest

    def sets(self):
        """Return a list of each disjoint set"""
        ret = defaultdict(list)
        for k, _ in self.parents.iteritems():
            ret[self[k]].append(k)
        return ret.values()

    
if __name__ == '__main__':

    # test
    uf = UnionFind()
    uf.union(0, 1)
    uf.union(2, 3)
    uf.union(3, 0)
    uf.union(4, 5)
    print uf.sets()
    assert uf.sets() == [[0, 1, 2, 3], [4, 5]]
    uf.union(3, 5)
    print uf.sets()
    assert uf.sets() == [[0, 1, 2, 3, 4, 5]]



#!/usr/bin/env python


try:
    from pyspark import SparkContext
except:
    print "### NO PYSPARK"
import json
import argparse
import sys
from digTokenizer.tokenizer import Tokenizer
from optparse import OptionParser
from digSparkUtil.dictUtil import as_dict, dict_minus
from digLshClustering import Hasher
from digLshClustering import Clusterer
from digLshClustering.clusterer.unionFind import UnionFind


from digSparkUtil.fileUtil import FileUtil

def testLSH(sc, inputFilename,outputFilename,configFilename,
             **kwargs):

    '''
    kwargs is a dictionary of inputs a sample input would look like
    options = {
               "file_format":"sequence",
               "data_type":"json",
               "numHashes":100 ,
               "numItemsInBand": 10,
               "computeSimilarity": True,
               "threshold":0.8,
               "base":"saam-city.json",
               "topk":3,
               "candidatesName":"candidates")
               }
    '''
    futil = FileUtil(sc)


    #Tokenize
    ######################
    rdd_input = futil.load_file(inputFilename, file_format=kwargs['file_format'], data_type="json")
    rdd_input.setName('rdd_input')

    tokenizer = Tokenizer(configFilename, **kwargs)
    rdd_tokenized = tokenizer.perform(rdd_input)

    futil = FileUtil(sc)
    outOptions = {}

    #you can save the tokens file here by using
    #futil.save_file(rdd_tokenized,outputFilename,file_format='sequence',data_type='json',**outOptions)

    #Hashing
    #######################

    hasher = Hasher(**kwargs)
    rdd_minHashes = hasher.perform(rdd_tokenized)
    rdd_minHashes.setName('rdd_minhashes')
    #futil.save_file(rdd_minHashes,outputFilename,file_format='sequence',data_type='json',**outOptions)


    #clustering
    #########################
    clusterer = Clusterer(**kwargs)
    rdd_clusters = clusterer.perform(rdd_minHashes)
    #futil.save_file(rdd_clusters,outputFilename,file_format='text',data_type='json',**outOptions)

    #unionfind
    #########################
    unionFind = UnionFind(**kwargs)
    rdd_unionfind = unionFind.perform(rdd_clusters)

    # SAVE DATA
    futil.save_file(rdd_unionfind,outputFilename,file_format='text',data_type='json',**outOptions)

def main():



    parser = argparse.ArgumentParser()

    parser.add_argument('-i','--inputFile', required=True)
    parser.add_argument('-o','--output_dir', required=True)
    parser.add_argument('--config', default=None, required=True)
    parser.add_argument('--file_format', default='sequence', choices=('text', 'sequence'))

    parser.add_argument("-n","--numHashes",type=int,help="number of minhashes", default=100)
    parser.add_argument("-b","--numItemsInBand",type=int,help="number of items in each band", default=10)

    parser.add_argument("-s","--computeSimilarity",action="store_true",help="compute similarity", default=False)
    parser.add_argument("-m", "--saveMinhashes", action="store_true", default=False, help="save minhashes")

    parser.add_argument("-t","--threshold",type=float,help="similarity threshold", default=0.0)
    parser.add_argument("-e","--base",help="base file", default="")
    parser.add_argument("-k","--topk", type=int,help="top n matches", default=3)
    parser.add_argument("-z","--candidates_name",help="name for json element for matching candidates", default="candidates")

    args = parser.parse_args()
    kwargs = dict_minus(as_dict(args), "inputFile", "output_dir", "config")

    print kwargs,type(kwargs)
    sc = SparkContext(appName="LSH-HASHER")
    testLSH(sc,args.inputFile,args.output_dir,args.config,**kwargs)

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

import sys
import hashlib

numHashes = 100
numItemsInBand = 10
keyPrefix = ""

class Signature(object):
    """Signature Base class."""

    def __init__(self, dim):
        self.dim = dim
        self.hashes = self.hash_functions()

    def hash_functions(self):
        """Returns dim different hash functions"""
        pass

    def sign(self, object):
        """Return the signature for object s"""
        pass


class MinHashSignature(Signature):
    """Creates signatures for sets/tuples using minhash."""

    def hash_functions(self):
        """Return dim different hash functions"""
        def hash_factory(n):
            return lambda x: hashlib.sha1(str(n) + x).hexdigest()
             #return lambda x: hash("salt" + unicode(n) + unicode(x) + "salt")

        return [ hash_factory(_) for _ in range(self.dim) ]

    def sign(self, s):
        """Returns minhash signature for set s"""
        sig = [ float("inf") ] * self.dim
        for hash_ix, hash_fn in enumerate(self.hashes):
            sig[hash_ix] = min(hash_fn(value) for value in s)
            #sig[hash_ix] = min(self.callHashFunc(hash_fn, value) for value in s)
        return sig

    def callHashFunc(self, func, value):
        value = func(value)


class LSH(object):
    """Locality sensitive hashing.  Uses a banding approach to hash
    similar signatures to the same buckets."""
    def __init__(self, n, r, t):
        self.n = n
        self.r = r
        self.b = int(self.n / self.r)
        if t:
            self.t = t
        else:
            self.t = self.get_threshold()
        #print "Setting threshold=" + str(self.t)

    def hash(self, sig):
        """Generate hashvals for this signature"""
        for band in zip(*(iter(sig),) * self.r):
            #band = r values from sig at a time
            yield ''.join(list(band))
            #yield hashlib.sha1("salt" + unicode(band) + "tlas").hexdigest()
            #yield hash("salt" + unicode(band) + "tlas")


    def get_threshold(self):
        r = self.r
        b = self.b
        return (1. / b) ** (1. / r)

    def get_n_bands(self):
        return self.b

hasher = LSH(numHashes, numItemsInBand, None)
signer = MinHashSignature(numHashes)
numBands = numHashes/numItemsInBand
separator = "\t"
for line in sys.stdin:
    # remove leading and trailing whitespace
    line = line.strip()
    idx = line.find("\t")
    if idx != -1:
        key = line[0:idx]
        data = line[idx+1:]
        if len(data) > 0:
            #print "Data:" + data
            tokens = data.split(separator)
            if len(tokens) > 0:
                #print "Adding tokens: " + str(tokens)
                minHashSig = signer.sign(tokens)
                lshSig = list(hasher.hash(minHashSig))
                minOut = '\t'.join(minHashSig)

                for i in range(0, numBands):
                    print (str(i).zfill(3) + ":" + lshSig[i] + separator +
                           keyPrefix + key + separator + minOut)

exit(0)



#!/usr/bin/env python

import sys
import hashlib

# input comes from STDIN (standard input)
for line in sys.stdin:
    # remove leading and trailing whitespace
    line = line.strip()
    if len(line) > 0:
        #input is the LSH file:
        #   lshkey  key minhash
        (lshkey, key, minhash) = line.split("\t", 2)
        cluster_id = hashlib.sha1(minhash).hexdigest()
        lshkey_cluster_id = lshkey + "$$$$" + cluster_id
        print "%s\t%s" % (lshkey_cluster_id, minhash)

exit(0)




#!/usr/bin/env python

import sys

# input comes from STDIN (standard input)
for line in sys.stdin:
    # remove leading and trailing whitespace
    line = line.strip()
    (x, y, rest) = line.split("\t", 2)
    key = x + "$$$$" + y
    print key + "\t" + rest




__author__ = 'dipsy'
import os
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

def write_tokens(tokens, sep):
    outStr = ""
    usep = ""
    for token in tokens:
        outStr = outStr + usep + str(token)
        usep = sep
    return outStr


def get_int_list(list1):
    ret = []
    for e in list1:
        if(len(e) > 0) and e != 'None':
            ret.append(int(e))
    return ret

def compute_list_similarity(list1, list2):
    similarity = float(len(set(list1) & set(list2)))/float(len(set(list1)))
    return similarity

def sort_csv_file(filename, columnArr, delim):
    #This a hack to fix a bug with sorting multiple files using this csvsort library.
    TMP_DIR = '.csvsort.%d' % os.getpid()
    if not os.path.exists(TMP_DIR):
        os.mkdir(TMP_DIR)
    from csvsort import csvsort
    if os.stat(filename).st_size > 0:
        csvsort(filename, columnArr, delimiter=delim, has_header=False)
        try:
            for the_file in os.listdir(TMP_DIR):
                file_path = os.path.join(TMP_DIR, the_file)
                try:
                    if os.path.isfile(file_path):
                        os.unlink(file_path)
                except:
                    pass
            os.rmdir(TMP_DIR)
        except OSError:
            pass

class Searcher:
    def __init__(self, filename):
        self.f = open(filename, 'rb')
        self.f.seek(0,2)
        self.length = self.f.tell()

    def readToEndOfLineAfterPos(self, p):
        while p >= 0:
            self.f.seek(p)
            if self.f.read(1) == '\n': break
            p -= 1
        if p < 0: self.f.seek(0)
        return p

    def seekBackToFindStr(self, p, string, key=lambda val: val):
        delta = 100
        prev_p = -1
        while p >= 0:
            p = p - delta
            p = self.readToEndOfLineAfterPos(p)
            line = (self.f.readline())
            #print "seek got:", line, ":", string, ":", p
            if key(line) != string:
                p = prev_p
                self.readToEndOfLineAfterPos(p)
                break
            prev_p = p
        if p < 0: self.f.seek(0)
        return p

    def find(self, string,  key=lambda val: val):
        low = 0
        high = self.length
        while low < high:
            mid = (low+high)//2
            p = mid
            self.readToEndOfLineAfterPos(p)
            line = self.f.readline()
            keyVal = key(line)
            #print '--', low, high, mid, keyVal
            if keyVal == string:
                low = self.seekBackToFindStr(mid, string, key)
                break

            if keyVal < string:
                low = mid+1
            else:
                high = mid

        self.readToEndOfLineAfterPos(low)

        result = [ ]
        while True:
            line = self.f.readline()
            keyVal = key(line)
            if not line or not keyVal == string: break
            if line[-1:] == '\n': line = line[:-1]
            #print "Append: " + line + "\n"
            result.append(line)
        return result



from setuptools import setup, find_packages
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


config = {
    'name': 'digLshClustering',
    'description': 'code to cluster based on lsh algorithm',
    'author': 'Dipsy Kapoor',
    'url': 'https://github.com/usc-isi-i2/dig-lsh-clustering.git',
    'download_url': 'https://github.com/usc-isi-i2/dig-lsh-clustering.git',
    'author_email': 'dipsykapoor@gmail.com',
    'install_requires': ['nose2',
                         'digSparkUtil',
                         'jq',
                         'digTokenizer'],
    'version':'0.1.11',
    'packages': find_packages(exclude=['digLshClustering.tests','digLshClustering.gen_int_input']),
    'scripts': []
}

setup(**config)



#!/usr/bin/env python

from pyspark import SparkContext

from optparse import OptionParser
from tokenizer.tokenizer import Tokenizer
from hasher.hasher import Hasher
from clusterer.clusterer import Clusterer
import json

if __name__ == "__main__":
    """
        Usage: tokenizer.py [input] [output] [reducer:feature_name]...
    """
    sc = SparkContext(appName="LSH")

    usage = "usage: %prog [options] input config output"
    parser = OptionParser()
    parser.add_option("-r", "--separator", dest="separator", type="string",
                      help="field separator", default="\t")
    parser.add_option("-d", "--type", dest="data_type", type="string",
                      help="input data type: csv/json", default="csv")
    parser.add_option("-i", "--inputformat", dest="inputformat", type="string",
                      help="input file format: text/sequence", default="text")
    parser.add_option("-n", "--numHashes", dest="numHashes", type="int",
                      help="number of minhashes", default=100)
    parser.add_option("-b", "--numItemsInBand", dest="numItemsInBand", type="int",
                      help="number of items in each band", default=10)
    parser.add_option("-s", "--computeSimilarity", action="store_true",
                      dest="computeSimilarity", default=False, help="compute similarity")
    parser.add_option("-j", "--computeIdenticalClusters", action="store_true",
                      dest="computeIdenticalClusters", default=False, help="compute identical clusters")
    parser.add_option("-t", "--threshold", type="float",
                      dest="threshold", default=0.0, help="similarity threshold")
    parser.add_option("-e", "--base", dest="base", type="string",
                      help="base file", default="")
    parser.add_option("-c", "--baseConfig", dest="baseConfig", type="string",
                      help="base config file", default="")
    parser.add_option("-o", "--outputformat", dest="outputformat", type="string",
                      help="output file format: text/sequence", default="text")
    parser.add_option("-x", "--numPartitions", dest="numPartitions", type="int",
                      help="number of partitions", default=10)
    parser.add_option("-y", "--outputtype", dest="outputtype", type="string",
                      help="output type: csv/json", default="json")
    parser.add_option("-k", "--topk", dest="topk", type="int",
                      help="top n matches", default=3)
    parser.add_option("-z", "--candidatesName", dest="candidates_name", type="string",
                        help="name for json element for matching candidates", default="candidates")

    (c_options, args) = parser.parse_args()
    print "Got options:", c_options
    inputFilename = args[0]
    configFilename = args[1]
    outputFilename = args[2]

    tokenizer = Tokenizer(configFilename, c_options)
    if c_options.inputformat == "text":
        rdd = tokenizer.tokenize_text_file(sc, inputFilename, c_options.data_type)
    else:
        rdd = tokenizer.tokenize_seq_file(sc, inputFilename, c_options.data_type)
    rdd.partitionBy(c_options.numPartitions)

    hasher = Hasher(c_options.numHashes, c_options.numItemsInBand, c_options.computeSimilarity)
    input_lsh_rdd = hasher.compute_hashes(rdd)

    clusterer = Clusterer(c_options.numPartitions,
                          c_options.computeSimilarity, c_options.threshold)

    if len(c_options.base) > 0:
        if len(c_options.baseConfig) > 0:
            tokenizer = Tokenizer(c_options.baseConfig, c_options)
        if c_options.inputformat == "text":
            base_rdd = tokenizer.tokenize_text_file(sc, c_options.base, c_options.data_type)
        else:
            base_rdd = tokenizer.tokenize_seq_file(sc, c_options.base, c_options.data_type)
        base_rdd.partitionBy(c_options.numPartitions)

        base_lsh_rdd = hasher.compute_hashes(base_rdd)
        result = clusterer.compute_clusters_with_base(input_lsh_rdd, base_lsh_rdd)
    else:
        if c_options.computeIdenticalClusters is True:
            (key_clusterids, result) = clusterer.compute_identical_clusters(input_lsh_rdd)
        else:
            result = clusterer.compute_clusters(input_lsh_rdd)

    if c_options.outputtype == "json":
        result = clusterer.output_json(result, c_options.topk, c_options.candidates_name)
    else:
        result = clusterer.output_csv(result, c_options.topk, c_options.separator)

    if c_options.outputformat == "text":
        if c_options.computeIdenticalClusters is True:
            key_clusterids.saveAsTextFile(outputFilename + "-key-clusterids")
        result.saveAsTextFile(outputFilename)
    else:
        if c_options.computeIdenticalClusters is True:
            key_clusterids.mapValues(lambda x, y: json.dumps(x)).saveAsSequenceFile(outputFilename + "-key-clusterids")
        result.mapValues(lambda x, y: json.dumps(x)).saveAsSequenceFile(outputFilename)



__author__ = 'dipsy'

import sys

from hasher.lsh import IntegerLSH, IntegerMinHashSignature


numHashes = 100
numItemsInBand = 10
keyPrefix = ""

hasher = IntegerLSH(numHashes, numItemsInBand, None)
signer = IntegerMinHashSignature(numHashes)

def write_tokens(tokens, sep):
    outStr = ""
    usep = ""
    for token in tokens:
        outStr = outStr + usep + str(token)
        usep = sep
    return outStr

def get_int_list(list1):
    ret = []
    for e in list1:
        if(len(e) > 0) and e != 'None':
            ret.append(int(e))
    return ret

numBands = numHashes/numItemsInBand
separator = "\t"
for line in sys.stdin:
    # remove leading and trailing whitespace
    line = line.strip()
    idx = line.find("\t")
    if idx != -1:
        key = line[0:idx]
        data = line[idx+1:]
        if len(data) > 0:
            #print "Data:" + data
            tokens = data.split(separator)
            if len(tokens) > 0:
                #print "Adding tokens: " + str(tokens)
                tokens = get_int_list(tokens)
                minHashSig = signer.sign(tokens)
                lshSig = list(hasher.hash(minHashSig))
                minOut = separator + write_tokens(minHashSig, separator)

                for i in range(0, numBands):
                    print (str(i).zfill(3) + ":" + lshSig[i] + separator + keyPrefix + key + minOut + "\n")

exit(0)



#!/usr/bin/env python

from optparse import OptionParser
import json

from pyspark import SparkContext
from lsh.lsh import MinHashSignature
from lsh.lsh import LSH


class Hasher:
    def __init__(self, **options):
        self.signer = MinHashSignature(options.get("numHashes",100))
        self.hasher = LSH(options.get("numHashes",100),options.get("numItemsInBand",10), None)
        self.save_min_hash = options.get("saveMinhashes",False)
        #print 'in hashing'
        print options.get("numHashes",100),options.get("numItemsInBand",10),options.get("saveMinhashes",False)


    def perform(self, rdd):
        #rdd = rdd.map(lambda (x,y) : (x,json.dumps(y)))
        return self.compute_hashes(rdd)

    def compute_hashes(self, data):
        return data.flatMap(lambda (x, y): self.compute_row_lsh(x, y))

    def compute_row_lsh(self, key, row):
        if len(row) > 0:
            min_hash_sig = self.signer.sign(row)
            #print "Sign:", row, " -> ", min_hash_sig
            #print min_hash_sig
            if min_hash_sig is not None:
                lsh_sig = list(self.hasher.hash(min_hash_sig))
                if self.save_min_hash is False:
                    min_hash_sig = None
                for lsh_val in lsh_sig:
                    #print lsh_val, (key, min_hash_sig)
                    yield lsh_val, (key, min_hash_sig)


if __name__ == "__main__":
    """
        Usage: hasher.py [input] [output]
    """
    sc = SparkContext(appName="LSH-HASHER")

    usage = "usage: %prog [options] input output"
    parser = OptionParser()
    parser.add_option("-n", "--numHashes", dest="numHashes", type="int",
                      help="number of minhashes", default=100)
    parser.add_option("-b", "--numItemsInBand", dest="numItemsInBand", type="int",
                      help="number of items in each band", default=10)
    parser.add_option("-s", "--saveMinhashes", action="store_true",
                      dest="saveMinhashes", default=False, help="save minhashes")

    (c_options, args) = parser.parse_args()
    print "Got options:", c_options
    inputFilename = args[0]
    outputFilename = args[1]


    hasher = Hasher(c_options)
    rdd = sc.sequenceFile(inputFilename).mapValues(lambda x: json.loads(x))
    result = hasher.compute_hashes(rdd)
    result.mapValues(lambda x: json.dumps(x)).saveAsSequenceFile(outputFilename)

